<?php
/**
 * Sisow MisterCash
 * @class SisowMC
 *
 * @version 2.1
 * @copyright Sisow, august 2011
 * @package shopp
 * @since 1.1.7
 * @subpackage SisowMC
 * 
 * $Id: sisowmc.php 211 2011-08-16 19:44:17Z rba $
 *
 **/

require_once(SHOPP_PATH."/core/model/XML.php");

class SisowMC extends GatewayFramework implements GatewayModule {          

	var $secure = false;
	var $url = 'https://www.sisow.nl/Sisow/iDeal/RestHandler.ashx/';
	var $buttonurl = '/gateways/sisow/bcmc.png';

	function __construct() {
		parent::__construct();
		$this->setup('merchantid', 'merchantkey', 'testmode');
		$this->buttonurl = SHOPP_PLUGINURI . $this->buttonurl;

		// Setup extra checkout inputs here so they are available
		// any time the gateway is active
		//add_filter('shopp_checkout_gateway_inputs', array(&$this, 'sisow_inputs'));
		add_filter('shopp_checkout_submit_button', array(&$this, 'sisow_button'), 10, 3);
		add_action('shopp_txn_update', array(&$this, 'sisow_update'));
	}
	
	function actions() { 
		//add_action('shopp_init_checkout', array(&$this, 'sisow_checkout'));
		add_action('shopp_checkout_processed', array(&$this, 'sisow_processed'));
		add_action('shopp_remote_payment', array(&$this, 'sisow_payment'));
		add_action('shopp_process_order', array(&$this, 'sisow_order'));
	}
	
	function settings() {
		$this->ui->text(0, array(
			'name' => 'merchantid',
			'value' => $this->settings['merchantid'],
			'size' => 30,
			'label' => __('Sisow MerchantID.', 'Shopp')
		));
		
		$this->ui->text(0, array(
			'name' => 'merchantkey',
			'value' => $this->settings['merchantkey'],
			'size' => 40,
			'label' => __('Sisow MerchantKey.','Shopp')
		));
		
		$this->ui->checkbox(0, array(
			'name' => 'testmode',
			'label' => __('Test', 'Shopp'),
			'checked' => $this->settings['testmode']
		));
	}
	
	//function sisow_checkout() {
	//	add_filter('shopp_checkout_submit_button', array(&$this, 'sisow_button'), 10, 3);
	//}
	
	function sisow_processed() {
		global $Shopp;
		
		if (strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE') > 0 && !isset($_POST['process_x'])) return;
		
		$this->Order->Billing->cardtype = "MisterCash";
		
		$posts = array();
		$posts['payment'] = 'mistercash';
		$posts['merchantid'] = $this->settings['merchantid'];
		$posts['purchaseid'] = mktime();
		//$posts['notifyurl'] = shoppurl(array('rmtpay' => 'process'), 'confirm-order');
		$posts['returnurl'] = shoppurl(array('rmtpay' => 'process'), 'confirm-order'); // . '&pmt=mc&s=' . $Shopp->Shopping->session;
		$posts['callback'] = shoppurl(array('_txnupdate' => 'mc'), 'checkout') . '&s=' . $Shopp->Shopping->session;
		$description = array();
		foreach($Shopp->Order->Cart->contents as $i => $Item) {
			$description[] = $Item->quantity . " x " . $Item->name . ((!empty($Item->option->label)) ? ' ' . $Item->option->label : '');
		}
		$posts['description'] = join(", ", $description);
		if ($this->settings['testmode'] == 'on') {
			$posts['issuerid'] = '99';
		}
		$posts['amount'] = round(floatvalue($Shopp->Order->Cart->Totals->total) * 100);
		$posts['sha1'] = sha1($posts['purchaseid'] . $posts['purchaseid'] . $posts['amount'] . $this->settings['merchantid'] . $this->settings['merchantkey']);
		
		$posts['firstname']				= $Order->Customer->firstname;
		$posts['lastname']				= $Order->Customer->lastname;
		$posts['lc']					= $this->baseop['country'];
		$AddressType = "Shipping";
		// Disable shipping fields if no shipped items in cart
		if (empty($Order->Cart->shipped)) {
			$AddressType = "Billing";
			$posts['no_shipping'] = 1;
		}
		$posts['address_override'] 	= 1;
		$posts['address1']			= $Order->{$AddressType}->address;
		if (!empty($Order->{$AddressType}->xaddress))
			$posts['address2']		= $Order->{$AddressType}->xaddress;
		$posts['city']				= $Order->{$AddressType}->city;
		$posts['state']				= $Order->{$AddressType}->state;
		$posts['zip']				= $Order->{$AddressType}->postcode;
		$posts['country']			= $Order->{$AddressType}->country;
		$posts['phone']				= $Order->Customer->phone;
		
		$Response = $this->sisow_send('TransactionRequest', $this->encode($posts));

		if (empty($Response) || $this->sisow_error($Response)) {
			return false;
		}

		$url = urldecode($Response->content('issuerurl'));
		if (!empty($url)) {
			shopp_redirect($url);
		}
		return false;
	}
	
	function sisow_update() {
		global $Shopp;
		
/*$this->sisow_report('SisowMC sisow_update ' . $_GET['_txnupdate']);*/
		if (!isset($_GET['_txnupdate']) || $_GET['_txnupdate'] != 'mc') return false;
		$Shopp->resession($_GET['s']);
		$Shopp->Order = ShoppingObject::__new('Order', $Shopp->Order);
		$this->actions();
		$Shopping = &$Shopp->Shopping;
		// Couldn't load the session data
		if ($Shopping->session != $_GET['s']) {
			return new ShoppError("Session could not be loaded: {$_GET['s']}", false, SHOPP_TRXN_ERR);
		}
		else {
			new ShoppError("Sisow successfully loaded session: {$_GET['s']}", false, SHOPP_TRXN_ERR);
		}
		do_action('shopp_process_order');
		exit;
	}
	
	function sisow_payment() {
/*$this->sisow_report('SisowMC sisow_payment');*/
		do_action('shopp_process_order');
	}
	
	function sisow_order() {
		global $Shopp;
		
		$redirect = !isset($_GET['notify']) && !isset($_GET['callback']);

		if (empty($_GET['trxid'])) {
			return false;
		}
		
		$posts = array();
		$posts['merchantid'] = $this->settings['merchantid'];
		$posts['trxid'] = $_GET['trxid'];
		$posts['sha1'] = sha1($_GET['trxid'] . $this->settings['merchantid'] . $this->settings['merchantkey']);

		if (!$Shopp->Order->validate()) {
			new ShoppError(__('There is not enough customer information to process the order.', 'Shopp'), 'invalid_order', SHOPP_TRXN_ERR);
			if (!$redirect) exit;
			shopp_redirect(shoppurl(false, 'cart'));
			return false;
		}
		
		// Check for unique transaction id
		$Purchase = new Purchase($posts['trxid'], 'txnid');
		if (!empty($Purchase->id)) {
			if (SHOPP_DEBUG) new ShoppError(__('Order validation failed. Received duplicate transaction id: ', 'Shopp') . $posts['trxid'], 'duplicate_order', SHOPP_TRXN_ERR);
			if (!$redirect) exit;
			shopp_redirect(shoppurl(false, 'cart'));
			return false;
		}

		$Response = $this->sisow_send('StatusRequest', $this->encode($posts));

		if (empty($Response) || $this->sisow_error($Response)) {
			new ShoppError(__('Payment could not be confirmed, this order cannot be processed.', 'Shopp'), 'sisow_ideal_status_error', SHOPP_TRXN_ERR);
			if (!$redirect) exit;
			shopp_redirect(shoppurl(false,'cart'));
			return false;
		}
		$status = $Response->content('status');
		
		if ($status != 'Success') {
			new ShoppError(__('Payment could not be confirmed, this order cannot be processed.', 'Shopp'), 'sisow_ideal_status_error', SHOPP_TRXN_ERR);
			if (!$redirect) exit;
			shopp_redirect(shoppurl(false,'cart'));
			return false;
		}

		$Shopp->Order->transaction($posts['trxid'], 'CHARGED');
	}
	
	function sisow_error($Response) {
		if (empty($Response))
			return false;
		$errorcode = $Response->content('errorcode');
		if (!$errorcode)
			return false;
		$errormessage = $Response->content('errormessage');
		return new ShoppError($errormessage, 'sisow_gateway_error', SHOPP_TRXN_ERR, array('code' => $errorcode));
	}
		
	function sisow_send($meth, $data, $report=true) {
		$result = parent::send($data, $this->url . $meth);
		return new xmlQuery($result);
	}
		
		
	function sisow_button($tag=false, $options=array(), $attrs=array()) {
		//return '<input type="image" name="process" src="' . $this->buttonurl . '" id="checkout-button" alt="iDeal" width="75" height="75" ' . inputattrs($options, $attrs) . ' />';
		$tag[$this->settings['label']] = '<input type="image" name="process" src="' . $this->buttonurl . '" alt="MisterCash" width="75" height="75" ' . inputattrs($options, $attrs) . ' />';
		return $tag;
	}

	function sisow_report($message) {
		$postsp = array();
		$postsp['message'] = 'shopp: '.$message;
		$this->sisow_send('Report', $this->encode($postsp), false);
	}
} // END class SisowMC

?>